"""
-------------------------------------------------------------------------------------------

 Project : PAL or (NTSC) Colour Bar Generator
 Author  : Mike G7GTN 
 Version : 1.00
 Date    : 25/5/2016
 Python  : 3.4.2
 Purpose : To provide a very simple Colour Bar Generator with switchable user captions

-------------------------------------------------------------------------------------------

       Python & Pygame code for the article published in CQ-DATV 36

       Additional text rendering functions come from the included extension 

       https://github.com/cosmologicon/pygame-text

  Hardware:-

  (6) x Pushbutton tactile switches

  SW1 = Bar Select          GPIO 26  
  SW2 = Large Caption       GPIO 19
  SW3 = Caption             GPIO 16
  SW4 = Clear               GPIO 21  
  SW5 = Power down control  GPIO 12

  Installation Instructions:-
     
  Firstly if not already on your system you need to install the additional   
  python library called pygame. A website is available where you can find  
  tutorials and full syntax of all commands http://www.pygame.org/hifi.html    
  My version of Raspbian Jessie 2016-02-16 had this pre-installed via the    
  image file. This image file came from the official Raspberry pi foundation 
  web site https://www.raspberrypi.org/downloads/
          
  To select the required video standard and aspect ratio you have to edit      
  the boot config.txt file. In the zip file is a copy of my file for you     
  to see the couple of small changes I needed to make. This file needs to be
  placed in /boot after which you should repower the pi Zero module.                               

  sdtv_mode=0 NTSC
  sdtv_mode=1 Japanese NTSC
  sdtv_mode=2 PAL 
  sdtv_mode=3 Brazilian version of PAL

  sdtv_aspect=1 4:3
  sdtv_aspect=2 14:9                        
  sdtv_aspect=3 16:9

  GPIO Pins:-

  To be able to access the GPIO pins from within python you must run the 
  script as Sudo. Further programming information on the pi GPIO is                             
  available here https://sourceforge.net/p/raspberry-gpio-python/wiki/BasicUsage/                                 

  Power Down:-

  A power down function has been added to allow the unit to be safely shutdown
  with just a single button press. This will save your SD card from becoming
  potentially corrupted.

  To stop the script you may connect a USB Mouse and press either button this will 
  allow you to continue working on the pi. 

  Autorun: -

  To allow the bargen.py script to autostart on power up we need to edit a file
  from a terminal command prompt enter the below command: -

  sudo nano ~/.config/lxsession/LXDE-pi/autostart

  Add the following line to the end

  @sudo /usr/bin/python3 /home/pi/bargen.py

  Then press ctrl + x + y then enter

  Now when you reboot the Colour Bar Generator project will be automatically run for you.

"""

#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pygame
import ptext

try:
    import RPi.GPIO as GPIO
except RuntimeError:
    Print("bargen.py must be run under Sudo for GPIO pin access")
import time
import os
import sys
from sys import exit
from pygame.locals import *
from time import sleep

pygame.init()

pygame.mouse.set_visible(0)

# User Button I/O Setup
GPIO.setmode(GPIO.BCM)
GPIO.setup(12, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(16, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(19, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(20, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(21, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(26, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setwarnings(False)

# Our loaded colour bar images 
barimage1 = pygame.image.load("cbars/colourbars.png")
barimage2 = pygame.image.load("cbars/bars2.jpg")

# Set up some colours
black = (0, 0, 0)
white = (255, 255, 255)
blue = (0, 0, 255)
green = (0, 128, 0)
yellow = (255, 255, 0)

# Screen Dimensions
SCREENWIDTH = 768
SCREENHEIGHT = 576
#screen = pygame.display.set_mode((SCREENWIDTH, SCREENHEIGHT))
screen = pygame.display.set_mode((SCREENWIDTH, SCREENHEIGHT), pygame.FULLSCREEN)

# Start with default colour bar image
screen.blit(barimage1, (0, 0))
pygame.display.flip()

running = True
#barimage = 0
bars_page = 1						


# Font directory
ptext.FONT_NAME_TEMPLATE = "fonts/%s.ttf"

"""
 Functions called by push buttons on GPIO pins
"""

def powerdown(channel):
    try:
        os.system("sudo shutdown -h now")
    except:
        pass

def bars1():
    screen.blit(barimage1, (0, 0))
    pygame.display.flip()

def bars2():
    screen.blit(barimage2, (0, 0))
    pygame.display.flip() 

# Small caption
def captiontwo(channel):
    # Calculate these values depending on your own captions length
    # First number is across the screen, next number is down
    ptext.draw("G7GTN - TELEVISION", (73, 346), fontname="mode7", fontsize=56, alpha=0.9, owidth=1, color="white")
    ptext.draw("TALKBACK 144.750MHz", (64, 430), fontname="mode7", fontsize=56, alpha=0.9, owidth=1, color="white")
    pygame.display.flip()

# Large Caption
def showlargecaptionariblk(channel):
    ptext.draw("G7GTN", (72, 62), fontname="ariblk", fontsize=150, alpha=1, owidth=1.1, color="white")  # mode 7 fontsize=200 alpha = 0.9
    ptext.draw("PLYMOUTH SOUTH DEVON", (80, 246), fontname="ariblk", fontsize=40, alpha=1, owidth=1.1, color="orange")  # mode 7 fontsize=90
    ptext.draw("RX = Modified Sat Receiver", (87, 342), fontname="ariblk", fontsize=40, alpha=1, owidth=1, color="white", align="center")  # mode 7 fontsize=40
    ptext.draw("TX = Solent 1W + 18W PA", (87, 382), fontname="ariblk", fontsize=40, alpha=1, owidth=1, color="white", align="center")  # mode 7 fontsize=40
    ptext.draw("Talkback on 2M 144.750MHz", (87, 424), fontname="ariblk", fontsize=40, alpha=1, owidth=1, color="white", align="center")  # mode 7 fontsize=40
    pygame.display.flip()

# Need to detect which bar image on
def captionclear(channel):
    # Re-flip original colour bar graphic image
    screen.blit(barimage1, (0, 0))
    pygame.display.flip()

"""
 Monitor user pushbutton events
"""
GPIO.add_event_detect(12, GPIO.FALLING, callback = powerdown, bouncetime = 2000)
GPIO.add_event_detect(16, GPIO.FALLING, callback=captiontwo, bouncetime=2000)
GPIO.add_event_detect(19, GPIO.FALLING, callback=showlargecaptionariblk, bouncetime=2000)
GPIO.add_event_detect(21, GPIO.FALLING, callback=captionclear, bouncetime=2000)
#GPIO.add_event_detect(26, GPIO.FALLING, callback=selectbars, bouncetime=2000)
GPIO.add_event_detect(26,GPIO.FALLING)

while running:
   # Multi push select colour bar pattern
   if GPIO.event_detected(26):
      GPIO.remove_event_detect(26)
      now = time.time()
      count = 1                                   
      GPIO.add_event_detect(26,GPIO.RISING)
      while time.time() < now + 3:               
         if GPIO.event_detected(26):
            count +=1                            
            time.sleep(.20)                      
      if count == 1:
          bars2()
   # If not in fullscreen mode
   for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            print("End run colour bar generator :-)")
            GPIO.cleanup()
            sys.exit()

        # Any mouse button to quit
        if event.type == pygame.MOUSEBUTTONDOWN:          
            pygame.quit();               
            print("End run colour bar generator :-)")
            GPIO.cleanup()
            sys.exit()
            
if __name__ == "__main__":
    main()
